# include <QtGui>

# include "cifsclient.h"

cifsclient::cifsclient(QWidget *parent): QDialog(parent)
{
	setupUi(this);

	QTextCodec::setCodecForTr(QTextCodec::codecForName("UTF-8"));

	QTimer::singleShot(0, this, SLOT(smbclient_test()));
	QTimer::singleShot(0, this, SLOT(cifs_test()));

	/************************************************/
	/*            server.conf-Datei laden           */
	/************************************************/
	int i=0;
	QString line;
	QString verzeichnis=QCoreApplication::applicationDirPath();
	// Rot und Grün Icons einbinden
	iconPathGreen=verzeichnis+"/buttons/spot_on_green.png";
	iconPathRed=verzeichnis+"/buttons/spot_off_red.png";
	green.addFile(iconPathGreen);
	red.addFile(iconPathRed);

	verzeichnis.append("/server.conf");

	// Wenn server.conf nicht existiert -> Programm beenden
	QFileInfo test(verzeichnis);
	if(!test.exists())
		{
		check_server_conf();
		}

	QFile datei(verzeichnis);
	if (datei.open(QIODevice::ReadOnly | QIODevice::Text))
		{
		QApplication::setOverrideCursor(Qt::WaitCursor);
	
		QTextStream in(&datei);
		while(!in.atEnd())
			{
			line = in.readLine();
			if(!line.isEmpty())
				{
				ServerComboBox->insertItem(i, line);
				i++;
				}
			}
		}
	datei.close();
	QApplication::restoreOverrideCursor();

	// wenn ServerComboBox leer, da keine Einträge in der server.conf -> Programm beenden
	if(ServerComboBox->currentIndex()==-1)
		check_server_conf();

	connect(ServerLoadButton, SIGNAL(clicked()), this, SLOT(load_Shares()));

	connect(&smbclient, SIGNAL(readyReadStandardOutput()), this, SLOT(smbclientAusgabe()));
	connect(&smbclient, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(smbclientProcBeendet(int, QProcess::ExitStatus)));

	connect(&mountProc, SIGNAL(readyReadStandardOutput()), this, SLOT(mountProcAusgabe()));
	connect(&mountProc, SIGNAL(readyReadStandardError()), this, SLOT(mountProcAusgabe()));
	connect(&mountProc, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(mountProcBeendet(int, QProcess::ExitStatus)));

	connect(&checkMountedProc, SIGNAL(readyReadStandardOutput()), this, SLOT(checkMountedAusgabe()));
	connect(&checkMountedProc, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(checkMountedBeendet()));

//	connect(ServerComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(clear_tableWidget()));

	// Timer für den Touch Befehl starten
	QTimer::singleShot(300000, this, SLOT(send_touch()));

}


/* ------------------- MAIN Funktion END ------------------- */



/********************************/
/*      Load Shares             */
/********************************/
void cifsclient::load_Shares()
{
	QString selectedServer;
	QString username;
	QString password;

	selectedServer = ServerComboBox->currentText();
	username = UsernameLineEdit->text();
	password = PasswordLineEdit->text();

	if(username.isEmpty() || password.isEmpty())
		{
		font.setPointSize(10);
		QMessageBox msgBox;
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.setFont(font);
		msgBox.setIcon(QMessageBox::Critical);
		msgBox.setWindowTitle("cifs-client");
		msgBox.setText(tr("<b>Fehler: Sie haben keinen Benutzernamen und/oder ein Paßwort angegeben!</b><br><br>"
			"Bitte füllen Sie die Felder username und password aus."));
		msgBox.exec();
		return;
		}

	QString user_password_combo;
	user_password_combo = username+"%"+password;

	QStringList arg;
	arg << "-g" << "-U" << user_password_combo << "-L" << selectedServer;
	QApplication::setOverrideCursor(Qt::WaitCursor);
	smbclient.start("smbclient", arg);
	smbclient.waitForFinished();
}


/*************************************************/
/* Ausgabe von smbclient - Ermitteln der Shares  */
/*************************************************/
void cifsclient::smbclientAusgabe()
{
	text_standard_output = "";
	QByteArray data= smbclient.readAllStandardOutput();
	text_standard_output = QString::fromLocal8Bit(data);

	smbclient_ausgabe.clear();

	if(!text_standard_output.isEmpty())
		{
		int pos1=0;
		int pos2=0;
		int lastpos=0;
		QString singleLine;

		while(pos1!=-1)
			{
			pos1=text_standard_output.indexOf("Disk",lastpos);
			lastpos=pos1+1;
			pos2=text_standard_output.indexOf("\n",lastpos);
			lastpos=pos2+1;

			if(pos1!=-1)
				{
				singleLine=text_standard_output.mid(pos1,pos2-pos1);

				if(singleLine.contains("$")) {}
				else if(singleLine.contains("Disk"))
					{
					int share_pos1;
					int share_pos2;
					share_pos1=singleLine.indexOf("|");
					share_pos2=singleLine.lastIndexOf("|");
					QString temp;
					temp = singleLine.mid(share_pos1+1,share_pos2-share_pos1-1);
					smbclient_ausgabe << temp;
					}
				}
			}
		}
	
}

/*************************************************/
/*          smbclient Prozess beendet            */
/*************************************************/
void cifsclient::smbclientProcBeendet(int exitCode, QProcess::ExitStatus exitStatus)
{
	QApplication::restoreOverrideCursor();
	signalMapper = new QSignalMapper(this);

	if (exitStatus==QProcess::CrashExit)
		{
		tableWidget->clear();
		tableWidget->setRowCount(1);
		tableWidget->setColumnCount(1);
		tableWidget->horizontalHeader()->resizeSection(0,700);
		tableWidget->setHorizontalHeaderItem(0, new QTableWidgetItem("Info"));
		tableWidget->setItem(0,0, new QTableWidgetItem("ERROR: SMBCLIENT CRASHED"));
		}
	else if(exitCode==1)
		{
		tableWidget->clear();
		tableWidget->setRowCount(1);
		tableWidget->setColumnCount(1);
		tableWidget->horizontalHeader()->resizeSection(0,700);
		tableWidget->setHorizontalHeaderItem(0, new QTableWidgetItem("Info"));
		tableWidget->setItem(0,0, new QTableWidgetItem(text_standard_output));
		}
	else	{
		// Um die Shareliste zu erstellen, zuerst das tableWidget löschen
		// und sicherstellen, daß wirklich alle Buttons von vorhergenden
		// Sharelisten gelöscht werden. Das wird durch den Slot clear_tableWidget erreicht.
		// Nach dem Slot clear_tableWidget wird der Slot create_Sharelist aufgerufen.
		clear_tableWidget();
		}
}


/*************************************************/
/*          mount-Button clicked                 */
/*************************************************/
void cifsclient::mountButtonClicked(int tabellenzeile)
{
	QList<QPushButton *> current_mountButton = tableWidget->findChildren<QPushButton *>();

	QString selectedServer;
	QString sharePath;
	QString mountDir;
	QString mountOptions;

	selectedServer = ServerComboBox->currentText();
	sharePath = "//"+selectedServer+"/"+tableWidget->item(tabellenzeile,1)->text();
	mountDir = tableWidget->item(tabellenzeile,2)->text();
	mountOptions = "username="+UsernameLineEdit->text()+",password="+PasswordLineEdit->text()+","+tableWidget->item(tabellenzeile,3)->text();

	current_mount_check=tableWidget->item(tabellenzeile,1)->text()+" on "+mountDir;

	if(current_mountButton[tabellenzeile]->text()=="mount")
		{
		QDir verzeichnistest(mountDir);
		if(!verzeichnistest.exists())
			{	
			// Bei fehlendem mount-Verzeichnis MessageBox
			font.setPointSize(12);
			QMessageBox msgBox;
			msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
			msgBox.setDefaultButton(QMessageBox::Yes);
			msgBox.setFont(font);
			msgBox.setIcon(QMessageBox::Information);
			msgBox.setWindowTitle("cifs-client: fehlendes mount-Verzeichnis");
			msgBox.setText("Das mount-Verzeichnis existiert nicht.<br>"
				"Soll es angelegt werden?");
			if(msgBox.exec() == QMessageBox::Yes)
				{
				verzeichnistest.mkdir(mountDir);
				}
			else	{
				return;
				}
			}

		QStringList arg;
		arg << "-t" << "cifs" << sharePath << mountDir << "-o" << mountOptions;
		QApplication::setOverrideCursor(Qt::WaitCursor);
		mountProc.start("mount", arg);
		}
	else if(current_mountButton[tabellenzeile]->text()=="umount")
		{
		QStringList arg;
		arg << mountDir;
		QApplication::setOverrideCursor(Qt::WaitCursor);
		mountProc.start("umount", arg);
		}
}

/*************************************************/
/*           Ausgabe von mount                   */
/*************************************************/
void cifsclient::mountProcAusgabe()
{
	QByteArray data= mountProc.readAllStandardOutput();
	QByteArray data2= mountProc.readAllStandardError();
	mountErrors = QString::fromLocal8Bit(data);
	mountErrors += QString::fromLocal8Bit(data2);
}


/*************************************************/
/*         mount Prozess beendet                 */
/*************************************************/
void cifsclient::mountProcBeendet(int exitCode, QProcess::ExitStatus exitStatus)
{
	QApplication::restoreOverrideCursor();
	if (exitStatus==QProcess::CrashExit)
		{

		}
	else if(exitCode==1)
		{
		// 1 bei nicht vorhandenem mount-Verzeichnis
		QString Fehlermeldung;
		Fehlermeldung = "mount meldete einen Fehler! (Error-Code:1)<br>";
		Fehlermeldung += "<pre>"+mountErrors+"</pre>";

		font.setPointSize(12);
		QMessageBox msgBox;
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.setDefaultButton(QMessageBox::Ok);
		msgBox.setFont(font);
		msgBox.setIcon(QMessageBox::Critical);
		msgBox.setWindowTitle("cifs-client: Fehler von mount");
		msgBox.setText(Fehlermeldung);
		msgBox.exec();
		}
	else if(exitCode==32)
		{
		/*	32 bei falschem Paßwort
			32 bei nicht vorhandenem Server
			32 bei nicht vorhandenem Sharenamen
		*/
		QString Fehlermeldung;
		Fehlermeldung = "mount meldete einen Fehler! (Error-Code:32)<br>";
		Fehlermeldung += "<pre>"+mountErrors+"</pre>";

		font.setPointSize(12);
		QMessageBox msgBox;
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.setDefaultButton(QMessageBox::Ok);
		msgBox.setFont(font);
		msgBox.setIcon(QMessageBox::Critical);
		msgBox.setWindowTitle("cifs-client: Fehler von mount");
		msgBox.setText(Fehlermeldung);
		msgBox.exec();
		}
	else	{
		checkMounted();
		}
}



/*************************************************/
/*           CheckMounted ausführen              */
/*************************************************/
void cifsclient::checkMounted()
{
	QApplication::setOverrideCursor(Qt::WaitCursor);

	QList<QPushButton *> current_mountButton = tableWidget->findChildren<QPushButton *>();

	int jobs;
	jobs=tableWidget->rowCount();

	int x;
	for(x=0;x<jobs;x++)
		{
		QString Sharename;
		Sharename = tableWidget->item(x,2)->text();
		is_mounted="";
		current_mount_check=tableWidget->item(x,1)->text()+" on "+tableWidget->item(x,2)->text();
		checkMountedProc.start("mount");
		checkMountedProc.waitForFinished();

		QTableWidgetItem *icon_item = new QTableWidgetItem;
		if(is_mounted=="Yes")
			{
			if(mountedShares.indexOf(Sharename) == -1)
				mountedShares << Sharename;

			icon_item->setIcon(green);
			tableWidget->setItem(x,4, icon_item);
			tableWidget->item(x,4)->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
			current_mountButton[x]->setText("umount");
			}
		else if(is_mounted=="No")
			{
			int pos;
			pos = mountedShares.indexOf(Sharename);
			if(pos!=-1)
				mountedShares.removeAt(pos);

			icon_item->setIcon(red);
			tableWidget->setItem(x,4, icon_item);
			tableWidget->item(x,4)->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
			current_mountButton[x]->setText("mount");
			}
		}
}


/*************************************************/
/*           Ausgabe checkMountedProc                */
/*************************************************/
void cifsclient::checkMountedAusgabe()
{
	text_standard_output = "";
	QByteArray data= checkMountedProc.readAllStandardOutput();
	text_standard_output = QString::fromLocal8Bit(data);

	if(text_standard_output.contains(current_mount_check))
		is_mounted="Yes";
	else
		is_mounted="No";
}


/*************************************************/
/*         checkMount Prozess beendet                 */
/*************************************************/
void cifsclient::checkMountedBeendet()
{
	QApplication::restoreOverrideCursor();
}

// Test, ob smbclient installiert ist
void cifsclient::cifs_test()
{
	QFileInfo test("/sbin/mount.cifs");
	if(!test.exists())
		{
		font.setPointSize(12);
		QMessageBox msgBox;
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.setFont(font);
		msgBox.setIcon(QMessageBox::Critical);
		msgBox.setWindowTitle("cifs-client");
		msgBox.setText(tr("<b>Fehler: mount.cifs nicht gefunden!</b><br><br>"
			"cifs-client benötigt mount.cifs zum Einhängen der Shares.<br>"
			"Bitte installieren Sie das Paket cifs-utils."));
		msgBox.exec();
		exit(EXIT_FAILURE);
		}
}


// Test, ob cifs installiert ist
void cifsclient::smbclient_test()
{
	QFileInfo test("/usr/bin/smbclient");
	if(!test.exists())
		{
		font.setPointSize(12);
		QMessageBox msgBox;
		msgBox.setStandardButtons(QMessageBox::Ok);
		msgBox.setFont(font);
		msgBox.setIcon(QMessageBox::Critical);
		msgBox.setWindowTitle("cifs-client");
		msgBox.setText(tr("<b>Fehler: smbclient nicht gefunden!</b><br><br>"
			"Um die Shares eines Windows-Servers abzufragen,<br>wird das Paket smbclient benötigt.<br>"
			"Bitte installieren Sie das Paket smbclient."));
		msgBox.exec();
		exit(EXIT_FAILURE);
		}
}

// Warnmeldung und Beenden des Programms bei fehlendem oder leerem server.conf-File
void cifsclient::check_server_conf()
{
	font.setPointSize(12);
	QMessageBox msgBox;
	msgBox.setStandardButtons(QMessageBox::Ok);
	msgBox.setFont(font);
	msgBox.setIcon(QMessageBox::Critical);
	msgBox.setWindowTitle("cifs-client");
	msgBox.setText(tr("<b>Fehler: Server Konfigurations-Datei nicht gefunden oder leer!</b><br><br>"
		"Bitte legen Sie im Programmverzeichnis eine Datei<br>"
		"namens server.conf an, in der Zeilenweise die<br>"
		"verfügbaren Windows-Server mittels Namen oder<br>"
		"I.P.-Adresse stehen."));
	msgBox.exec();
	exit(EXIT_FAILURE);

}


/*******************************************************************************/
/*  Sicherstellung das Buttons von vorhergenden Sharelisten gelöscht werden    */
/*******************************************************************************/
void cifsclient::clear_tableWidget()
{
	// Die Tabelle wird gelöscht
	tableWidget->setRowCount(0);
	tableWidget->setColumnCount(0);
	tableWidget->clear();

	// Die Buttons im tableWidget werden erfasst
	QList<QPushButton *> current_mountButton;
	current_mountButton = tableWidget->findChildren<QPushButton *>();

	// Sind Buttons vorhanden, also das tableWidget nicht leer, wird der Slot erneut aufgerufen und abgefragt
	if(!current_mountButton.isEmpty())
		{
		QTimer::singleShot(0, this, SLOT(clear_tableWidget()));
		}
	// enthält das tableWidget keine Buttons mehr, geht es weiter zur Erstellung der Shareliste
	else	{
		create_Sharelist();
		}
}


/*******************************************************************************/
/*      Eine Shareliste im tableWidget aus der smbclient-Ausgabe erstellen     */
/*******************************************************************************/
void cifsclient::create_Sharelist()
{
		int jobs;
		jobs=smbclient_ausgabe.size();

		tableWidget->setRowCount(smbclient_ausgabe.size());
		tableWidget->setColumnCount(5);
		tableWidget->horizontalHeader()->resizeSection(0,60);
		tableWidget->horizontalHeader()->resizeSection(1,194);
		tableWidget->horizontalHeader()->resizeSection(2,194);
		tableWidget->horizontalHeader()->resizeSection(3,194);
		tableWidget->horizontalHeader()->resizeSection(4,50);
		tableWidget->setHorizontalHeaderItem(0, new QTableWidgetItem("Butt"));
		tableWidget->setHorizontalHeaderItem(1, new QTableWidgetItem("Sharename"));
		tableWidget->setHorizontalHeaderItem(2, new QTableWidgetItem("Mountpoint"));
		tableWidget->setHorizontalHeaderItem(3, new QTableWidgetItem("Options"));
		tableWidget->setHorizontalHeaderItem(4, new QTableWidgetItem("Status"));

		int x=0;
		for(x=0;x<jobs;x++)
			{
			// mount-Buttons erstellen
			QString ButtonName;
			ButtonName.setNum(x);
			ButtonName = "mountButton"+ButtonName;

			QPushButton *mountButton = new QPushButton();
			mountButton->setObjectName(ButtonName);
//			mountButton->setText("mount");
			tableWidget->setCellWidget(x,0, mountButton);
			connect(mountButton, SIGNAL(clicked()), signalMapper, SLOT(map()));
			signalMapper->setMapping(mountButton, x);

			// Sharepoint einfügen
			tableWidget->setItem(x,1, new QTableWidgetItem(smbclient_ausgabe[x]));

			// Mountpoint einfügen
			QString mountpoint;
			mountpoint="/mnt/"+smbclient_ausgabe[x];
			tableWidget->setItem(x,2, new QTableWidgetItem(mountpoint));

			// Optionen
			tableWidget->setItem(x,3, new QTableWidgetItem("gid=1000,uid=1000,sec=ntlm"));

			}
		connect(signalMapper, SIGNAL(mapped(int)), this, SLOT(mountButtonClicked(int)));

		// checkMounted ausführen
		checkMounted();
}


/************************************************/
/*     touch-Befehl als ServerAlive-Signal      */
/************************************************/
void cifsclient::send_touch()
{
//	QString timestamp = QDateTime::currentDateTime().toString("dddd, dd.MM.yyyy hh:mm:ss");

	if(!mountedShares.isEmpty())
		{
		foreach(QString str, mountedShares)
			{
			QStringList arg;
			str = str+"/Server_Alive.txt";
			arg << "--no-create" << str;
			touchProc.start("touch", arg);
			touchProc.waitForFinished();
			}
		}
	QTimer::singleShot(300000, this, SLOT(send_touch()));
}



/**************************************************************
   Das Schließen der Applikation per close-Button oder X
   abfangen und gemountete Verzeichnisse überprüfen
 ***************************************************************/

void cifsclient::closeEvent(QCloseEvent *event)
{
	if(mountedShares.isEmpty())
		{
		event->accept();
		}
	else	{
		QString show_mounted_shares;
		show_mounted_shares = mountedShares.join("<br>");

		font.setPointSize(12);
		QMessageBox msgBox;
		msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
		msgBox.setDefaultButton(QMessageBox::Yes);
		msgBox.setFont(font);
		msgBox.setIcon(QMessageBox::Warning);
		msgBox.setWindowTitle("cifs-client: gemountete Verzeichnisse");
		msgBox.setText("Es sind noch Verzeichnisse gemounted.<br><pre>"
			+ show_mounted_shares +
			tr("</pre><br>Möchten Sie die Verzeichnisse aushängen<br>und das Programm beenden?"));
		if(msgBox.exec() == QMessageBox::Yes)
			{
			QString show_umounted_shares;
			foreach(QString str, mountedShares)
				{
				QApplication::setOverrideCursor(Qt::WaitCursor);
				QStringList arg;
				arg << str;
				mountProc.start("umount", arg);
				mountProc.waitForFinished();
				show_umounted_shares += "umount "+str+"<br>";
				QApplication::restoreOverrideCursor();
				}
			event->accept();
			}
		else	{
			event->ignore();
			}
		}
}


